@@ -1,5 +1,6 @@
-NbClust <-function(data, diss="NULL", distance = "euclidean", min.nc=2, max.nc=15, method = "ward", index = "all", alphaBeale = 0.1)
+NbClust <-function(data, diss=NULL, distance ="euclidean", min.nc=2, max.nc=15, method =NULL, index = "all", alphaBeale = 0.1)
 {
+    
     x<-0
     min_nc <- min.nc
     max_nc <- max.nc
@@ -33,60 +34,67 @@ NbClust <-function(data, diss="NULL", distance = "euclidean", min.nc=2, max.nc=1
   #                                                                                                                      #
   #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
   
-
- distance <- pmatch(distance, c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski","NULL"))
- if (is.na(distance)) 
- {
-	stop("invalid distance")
- }
+if(is.null(distance))
+  distanceM<-7
+if(!is.null(distance))
+  distanceM <- pmatch(distance, c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski"))
+  
+if (is.na(distanceM)) 
+{
+  stop("invalid distance")
+} 
     
- if (distance == -1) 
- {
-	stop("ambiguous distance")
- }
-    
- if (((distance==1) || (distance==2) || (distance==3) || (distance==4) || (distance==5) || (distance==6)) && (diss[1] != "NULL")) {
-	stop("ambiguous distance or dissimilarity matrix")
-	} 
-    if (distance == 1) {
+if(is.null(diss))
+{  
+      
+    if (distanceM == 1) 
+    {
     		md <- dist(jeu, method="euclidean")	# "dist" function computes and returns the distance matrix computed by using the specified distance measure to compute the distances between the rows of a data matrix.
-	}
-    if (distance == 2) {
+	  }
+    if (distanceM == 2) 
+      {
     		md <- dist(jeu, method="maximum")	
 	}
-    if (distance == 3) {
+    if (distanceM == 3) 
+      {
     		md <- dist(jeu, method="manhattan")	
 	}
-    if (distance == 4) {
+    if (distanceM == 4) 
+      {
     		md <- dist(jeu, method="canberra")	
 	}
-    if (distance == 5) {
+    if (distanceM == 5) 
+      {
     		md <- dist(jeu, method="binary")	
 	}
-    if (distance == 6) 
+    if (distanceM == 6) 
      {
     		md <- dist(jeu, method="minkowski")	
 	   }
 
-    if (distance == 7) 
-    {
+   if (distanceM == 7) 
+    {		  
+     stop("dissimilarity matrix and distance are both NULL")		
+    } 
+}
+
+if((!is.null(diss))&&((distanceM==1)||(distanceM==2)|| (distanceM==3)|| (distanceM==4)|| (distanceM==5)|| (distanceM==6)))
+    stop("dissimilarity matrix and distance are both not null")
+
+if((!is.null(diss))&&(distanceM==7))
+  md <- diss 
 		  
-      if(is.null(diss))
-      {
-		    stop("ambiguous distance or dissimilarity matrix")		
-      } 
-    		md <- diss 
-		  
-	  }
+
   
     #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
     #                                                                                                                      #
     #                                              Methods                                                                 #
     #                                                                                                                      #
     #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
-    
-  method <- pmatch(method, c("ward", "single", "complete", "average", 
-		"mcquitty", "median", "centroid", "kmeans"))
+  if(is.null(method))    
+    stop("method is NULL")
+  method <- pmatch(method, c("ward.D2", "single", "complete", "average", 
+		"mcquitty", "median", "centroid", "kmeans","ward.D"))
 
     res <- array(0, c(max_nc-min_nc+1,30))
     x_axis <- min_nc:max_nc
@@ -104,7 +112,7 @@ NbClust <-function(data, diss="NULL", distance = "euclidean", min.nc=2, max.nc=1
 	     stop("ambiguous method")
     if (method == 1) 
     {
-        hc<-hclust(md,method = "ward")      
+        hc<-hclust(md,method = "ward.D2")      
     }
     if (method == 2) 
     {
@@ -135,7 +143,13 @@ NbClust <-function(data, diss="NULL", distance = "euclidean", min.nc=2, max.nc=1
         hc<-hclust(md,method = "centroid")
 		 
 	  }
+    if (method == 9) 
+    {
+      hc<-hclust(md,method = "ward.D")
+  
+    }
 
+   
   
 
 #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
@@ -1087,13 +1101,14 @@ Indice.S <- function (d, cl)
     
     
 Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10, 
-    method = "ward", d = NULL, centrotypes = "centroids") 
+    method = "ward.D2", d = NULL, centrotypes = "centroids") 
 {
     GAP <- function(X, cl, referenceDistribution, B, method, 
         d, centrotypes) {
         simgap <- function(Xvec) {
             ma <- max(Xvec)
             mi <- min(Xvec)
+            set.seed(1)
             Xout <- runif(length(Xvec), min = mi, max = ma)
             return(Xout)
         }
@@ -1131,11 +1146,13 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
                 if (ClassNr == length(cl)) 
                   pp2 <- 1:ClassNr
                 else if (method == "k-means") 
+                { set.seed(1)
                   pp2 <- kmeans(Xnew, ClassNr, 100)$cluster
+                }
                 else if (method == "single" || method == "complete" || 
-                  method == "average" || method == "ward" || 
+                  method == "average" || method == "ward.D2" || 
                   method == "mcquitty" || method == "median" || 
-                  method == "centroid") 
+                  method == "centroid"|| method=="ward.D") 
                   pp2 <- cutree(hclust(dist(Xnew), method = method), 
                     ClassNr)
                 else stop("Wrong clustering method")
@@ -1158,12 +1175,15 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
             if (bb > 1) {
                 if (ClassNr == length(cl)) 
                   pp2 <- 1:ClassNr
-                else if (method == "k-means") 
+                else if (method == "k-means")
+                {
+                  set.seed(1)
                   pp2 <- kmeans(Xnew, ClassNr, 100)$cluster
+                }
                 else if (method == "single" || method == "complete" || 
-                  method == "average" || method == "ward" || 
+                  method == "average" || method == "ward.D2" || 
                   method == "mcquitty" || method == "median" || 
-                  method == "centroid") 
+                  method == "centroid"||method == "ward.D") 
                   pp2 <- cutree(hclust(dist(Xnew), method = method), 
                     ClassNr)
                 else stop("Wrong clustering method")
@@ -1324,7 +1344,7 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
  {  
       
 	   if (any(method == 1) || (method == 2) || (method == 3) || (method == 4) || 
-		  (method == 5) || (method == 6) || (method == 7)) 
+		  (method == 5) || (method == 6) || (method == 7)||(method == 9)) 
       {
 	      cl1 <- cutree(hc, k=nc)
 	      cl2 <- cutree(hc, k=nc+1)
@@ -1346,17 +1366,22 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
 	   
   	if (method == 8) 
     {
+      set.seed(1)
 		  cl2 <- kmeans(jeu,nc+1)$cluster
+      set.seed(1)
 		  clmax <- kmeans(jeu,max_nc)$cluster
       if (nc > 2)
 		  {
+        set.seed(1)
 		    cl1 <- kmeans(jeu,nc)$cluster
 		    clall <- cbind(cl1, cl2)
+        set.seed(1)
 		    cl0 <- kmeans(jeu,nc-1)$cluster
 		    clall1 <- cbind(cl0, cl1, cl2)
 		  }
 	    if (nc == 2)
 		  {
+        set.seed(1)
 	      cl1 <- kmeans(jeu,nc)$cluster
 		    clall <- cbind(cl1, cl2)
 		    cl0 <- rep(1,nn)
@@ -1491,7 +1516,7 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
 	{
 	  
 	  if (method == 1) {
-		resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "ward", d = NULL, centrotypes = "centroids")
+		resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "ward.D2", d = NULL, centrotypes = "centroids")
 		}
 	  if (method == 2) {
 	    	resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "single", d = NULL, centrotypes = "centroids")
@@ -1511,6 +1536,9 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
 	  if (method == 7) {
 	    	resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "centroid", d = NULL, centrotypes = "centroids")
 		}
+		if (method == 9) {
+		  resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "ward.D", d = NULL, centrotypes = "centroids")
+		}
 	  if (method == 8) {
 		resultSGAP <- Indice.Gap(x=jeu, clall=clall, reference.distribution = "unif", B = 10, method = "k-means", d = NULL, centrotypes = "centroids")
 		}
@@ -2237,11 +2265,14 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
       ########################## The Best partition    ###################
     
         if (any(method == 1) || (method == 2) || (method == 3) || (method == 4) || 
-          (method == 5) || (method == 6) || (method == 7))         
+          (method == 5) || (method == 6) || (method == 7)||(method == 9))         
             partition<- cutree(hc, k=j)
     
         else
+        {
+            set.seed(1)
             partition<-kmeans(jeu,j)$cluster
+        }
     
     }
     
@@ -2253,13 +2284,15 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
         ||(indice==28)||(indice==30))
     {
       if (any(method == 1) || (method == 2) || (method == 3) || (method == 4) || 
-            (method == 5) || (method == 6) || (method == 7)) 
+            (method == 5) || (method == 6) || (method == 7) || (method == 9)) 
       
         partition<- cutree(hc, k=best.nc)
       
       else
+      {
+        set.seed(1)
         partition<-kmeans(jeu,best.nc)$cluster
-      
+      }
             
     }
       
